@@ -3,7 +3,7 @@
 
 # This file is part of butterfly
 #
-# butterfly Copyright (C) 2015  Florian Mounier
+# butterfly Copyright(C) 2015-2017 Florian Mounier
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -20,7 +20,11 @@
 import tornado.options
 import tornado.ioloop
 import tornado.httpserver
-import tornado_systemd
+try:
+    from tornado_systemd import SystemdHTTPServer as HTTPServer
+except ImportError:
+    from tornado.httpserver import HTTPServer
+
 import logging
 import webbrowser
 import uuid
@@ -40,6 +44,9 @@ tornado.options.define("unminified", default=False,
 
 tornado.options.define("host", default='localhost', help="Server host")
 tornado.options.define("port", default=57575, type=int, help="Server port")
+tornado.options.define("keepalive_interval", default=30, type=int,
+                       help="Interval between ping packets sent from server "
+                       "to client (in seconds)")
 tornado.options.define("one_shot", default=False,
                        help="Run a one-shot instance. Quit at term close")
 tornado.options.define("shell", help="Shell to execute at login")
@@ -48,8 +55,18 @@ tornado.options.define("cmd",
                        help="Command to run instead of shell, f.i.: 'ls -l'")
 tornado.options.define("unsecure", default=False,
                        help="Don't use ssl not recommended")
-tornado.options.define("login", default=True,
+tornado.options.define("i_hereby_declare_i_dont_want_any_security_whatsoever",
+                       default=False,
+                       help="Remove all security and warnings. There are some "
+                       "use cases for that. Use this if you really know what "
+                       "you are doing.")
+tornado.options.define("login", default=False,
                        help="Use login screen at start")
+tornado.options.define("pam_profile", default="", type=str,
+                       help="When --login=True provided and running as ROOT, "
+                       "use PAM with the specified PAM profile for "
+                       "authentication and then execute the user's default "
+                       "shell. Will override --shell.")
 tornado.options.define("force_unicode_width",
                        default=False,
                        help="Force all unicode characters to the same width."
@@ -64,6 +81,10 @@ tornado.options.define("generate_current_user_pkcs", default=False,
 tornado.options.define("generate_user_pkcs", default='',
                        help="Generate user pfx for client authentication "
                        "(Must be root to create for another user)")
+tornado.options.define("uri_root_path", default='',
+                       help="Sets the servier root path: "
+                       "example.com/<uri_root_path>/static/")
+
 
 if os.getuid() == 0:
     ev = os.getenv('XDG_CONFIG_DIRS', '/etc')
@@ -79,10 +100,10 @@ ssl_dir = os.path.join(butterfly_dir, 'ssl')
 
 if not os.path.exists(conf_file):
     try:
+        import butterfly
         shutil.copy(
             os.path.join(
-                os.path.abspath(os.path.dirname(__file__)),
-                'butterfly',
+                os.path.abspath(os.path.dirname(butterfly.__file__)),
                 'butterfly.conf.default'), conf_file)
         print('butterfly.conf installed in %s' % conf_file)
     except:
@@ -120,6 +141,9 @@ log = logging.getLogger('butterfly')
 host = options.host
 port = options.port
 
+if options.i_hereby_declare_i_dont_want_any_security_whatsoever:
+    options.unsecure = True
+
 
 if not os.path.exists(options.ssl_dir):
     os.makedirs(options.ssl_dir)
@@ -128,6 +152,7 @@ if not os.path.exists(options.ssl_dir):
 def to_abs(file):
     return os.path.join(options.ssl_dir, file)
 
+
 ca, ca_key, cert, cert_key, pkcs12 = map(to_abs, [
     'butterfly_ca.crt', 'butterfly_ca.key',
     'butterfly_%s.crt', 'butterfly_%s.key',
@@ -153,6 +178,10 @@ def read(file):
     with open(file, 'rb') as fd:
         return fd.read()
 
+def b(s):
+    return s.encode('utf-8')
+
+
 if options.generate_certs:
     from OpenSSL import crypto
     print('Generating certificates for %s (change it with --host)\n' % host)
@@ -162,6 +191,7 @@ if options.generate_certs:
         ca_pk = crypto.PKey()
         ca_pk.generate_key(crypto.TYPE_RSA, 2048)
         ca_cert = crypto.X509()
+        ca_cert.set_version(2)
         ca_cert.get_subject().CN = 'Butterfly CA on %s' % socket.gethostname()
         fill_fields(ca_cert.get_subject())
         ca_cert.set_serial_number(uuid.uuid4().int)
@@ -169,6 +199,21 @@ if options.generate_certs:
         ca_cert.gmtime_adj_notAfter(315360000)  # to 10y
         ca_cert.set_issuer(ca_cert.get_subject())  # Self signed
         ca_cert.set_pubkey(ca_pk)
+        ca_cert.add_extensions([
+            crypto.X509Extension(
+                b('basicConstraints'), True, b('CA:TRUE, pathlen:0')),
+            crypto.X509Extension(
+                b('keyUsage'), True, b('keyCertSign, cRLSign')),
+            crypto.X509Extension(
+                b('subjectKeyIdentifier'), False, b('hash'), subject=ca_cert),
+        ])
+        ca_cert.add_extensions([
+            crypto.X509Extension(
+                b('authorityKeyIdentifier'), False,
+                b('issuer:always, keyid:always'),
+                issuer=ca_cert, subject=ca_cert
+            )
+        ])
         ca_cert.sign(ca_pk, 'sha512')
 
         write(ca, crypto.dump_certificate(crypto.FILETYPE_PEM, ca_cert))
@@ -182,7 +227,23 @@ if options.generate_certs:
     server_pk = crypto.PKey()
     server_pk.generate_key(crypto.TYPE_RSA, 2048)
     server_cert = crypto.X509()
+    server_cert.set_version(2)
     server_cert.get_subject().CN = host
+    server_cert.add_extensions([
+        crypto.X509Extension(
+            b('basicConstraints'), False, b('CA:FALSE')),
+        crypto.X509Extension(
+            b('subjectKeyIdentifier'), False, b('hash'), subject=server_cert),
+        crypto.X509Extension(
+            b('subjectAltName'), False, b('DNS:%s' % host)),
+    ])
+    server_cert.add_extensions([
+        crypto.X509Extension(
+            b('authorityKeyIdentifier'), False,
+            b('issuer:always, keyid:always'),
+            issuer=ca_cert, subject=ca_cert
+        )
+    ])
     fill_fields(server_cert.get_subject())
     server_cert.set_serial_number(uuid.uuid4().int)
     server_cert.gmtime_adj_notBefore(0)  # From now
@@ -232,6 +293,7 @@ if (options.generate_current_user_pkcs or
     client_pk.generate_key(crypto.TYPE_RSA, 2048)
 
     client_cert = crypto.X509()
+    client_cert.set_version(2)
     client_cert.get_subject().CN = user
     fill_fields(client_cert.get_subject())
     client_cert.set_serial_number(uuid.uuid4().int)
@@ -295,11 +357,10 @@ else:
 from butterfly import application
 application.butterfly_dir = butterfly_dir
 log.info('Starting server')
-http_server = tornado_systemd.SystemdHTTPServer(
-    application, ssl_options=ssl_opts)
+http_server = HTTPServer(application, ssl_options=ssl_opts)
 http_server.listen(port, address=host)
 
-if http_server.systemd:
+if getattr(http_server, 'systemd', False):
     os.environ.pop('LISTEN_PID')
     os.environ.pop('LISTEN_FDS')
 
